@@ -19,6 +19,7 @@
 
 #define G_LOG_DOMAIN "GcalWindow"
 
+#include "css-code.h"
 #include "gcal-calendar-management-dialog.h"
 #include "gcal-calendar-popover.h"
 #include "config.h"
@@ -38,6 +39,8 @@
 #include "gcal-window.h"
 #include "gcal-year-view.h"
 
+#include "importer/gcal-import-dialog.h"
+
 #include <glib/gi18n.h>
 #include <libecal/libecal.h>
 
@@ -116,6 +119,7 @@ struct _GcalWindow
   GtkWidget          *views_switcher;
 
   GcalEventEditorDialog *event_editor;
+  GtkWidget          *import_dialog;
 
   DzlSuggestionButton *search_button;
 
@@ -156,6 +160,10 @@ struct _GcalWindow
   /* handler for the searh_view */
   gint                click_outside_handler_id;
 
+  /* CSS */
+  GtkCssProvider     *provider;
+  GtkCssProvider     *colors_provider;
+
   /* Window states */
   gint                width;
   gint                height;
@@ -313,6 +321,80 @@ save_geometry (GcalWindow *self)
   GCAL_EXIT;
 }
 
+static void
+recalculate_calendar_colors_css (GcalWindow *self)
+{
+  g_autoptr (GError) error = NULL;
+  g_autoptr (GList) calendars = NULL;
+  g_autofree gchar *new_css_data = NULL;
+  g_auto (GStrv) new_css_snippets = NULL;
+  GcalManager *manager;
+  GQuark color_id;
+  GList *l;
+  gint arr_length;
+  gint i = 0;
+
+  manager = gcal_context_get_manager (self->context);
+  calendars = gcal_manager_get_calendars (manager);
+  arr_length = g_list_length (calendars);
+  new_css_snippets = g_new0 (gchar*, arr_length + 2);
+  for (l = calendars; l; l = l->next, i++)
+    {
+      g_autofree gchar* color_str = NULL;
+      const GdkRGBA *color;
+      GcalCalendar *calendar;
+
+      calendar = GCAL_CALENDAR (l->data);
+
+      color = gcal_calendar_get_color (calendar);
+      color_str = gdk_rgba_to_string (color);
+      color_id = g_quark_from_string (color_str);
+
+      new_css_snippets[i] = g_strdup_printf (CSS_TEMPLATE, color_id, color_str);
+    }
+
+  new_css_data = g_strjoinv ("\n", new_css_snippets);
+  gtk_css_provider_load_from_data (self->colors_provider, new_css_data, -1, &error);
+
+  if (error)
+    g_warning ("Error creating custom stylesheet. %s", error->message);
+}
+
+static void
+load_css_providers (GcalWindow *self)
+{
+  g_autoptr (GFile) css_file = NULL;
+  g_autofree gchar *theme_name = NULL;
+  g_autofree gchar *theme_uri = NULL;
+  GdkScreen *screen;
+
+  screen = gtk_widget_get_screen (GTK_WIDGET (self));
+
+  /* Theme */
+  self->provider = gtk_css_provider_new ();
+  gtk_style_context_add_provider_for_screen (screen,
+                                             GTK_STYLE_PROVIDER (self->provider),
+                                             GTK_STYLE_PROVIDER_PRIORITY_APPLICATION + 1);
+
+  /* Retrieve the theme name */
+  g_object_get (gtk_settings_get_default (), "gtk-theme-name", &theme_name, NULL);
+  theme_uri = g_strconcat ("resource:///org/gnome/calendar/theme/", theme_name, ".css", NULL);
+
+  /* Try and load the CSS file */
+  css_file = g_file_new_for_uri (theme_uri);
+
+  if (g_file_query_exists (css_file, NULL))
+    gtk_css_provider_load_from_file (self->provider, css_file, NULL);
+  else
+    gtk_css_provider_load_from_resource (self->provider, "/org/gnome/calendar/theme/Adwaita.css");
+
+  /* Calendar olors */
+  self->colors_provider = gtk_css_provider_new ();
+  gtk_style_context_add_provider_for_screen (screen,
+                                             GTK_STYLE_PROVIDER (self->colors_provider),
+                                             GTK_STYLE_PROVIDER_PRIORITY_APPLICATION + 2);
+}
+
 
 /*
  * Callbacks
@@ -636,17 +718,34 @@ create_event_detailed_cb (GcalView *view,
   g_clear_object (&comp);
 }
 
+static void
+event_preview_cb (GcalEventWidget        *event_widget,
+                  GcalEventPreviewAction  action,
+                  gpointer                user_data)
+{
+  GcalWindow *self = GCAL_WINDOW (user_data);
+  GcalEvent *event;
+
+  switch (action)
+    {
+    case GCAL_EVENT_PREVIEW_ACTION_EDIT:
+      event = gcal_event_widget_get_event (event_widget);
+      gcal_event_editor_dialog_set_event (self->event_editor, event, FALSE);
+      gtk_window_present (GTK_WINDOW (self->event_editor));
+      break;
+
+    case GCAL_EVENT_PREVIEW_ACTION_NONE:
+    default:
+      break;
+    }
+}
+
 static void
 event_activated (GcalView        *view,
                  GcalEventWidget *event_widget,
                  gpointer         user_data)
 {
-  GcalWindow *window = GCAL_WINDOW (user_data);
-  GcalEvent *event;
-
-  event = gcal_event_widget_get_event (event_widget);
-  gcal_event_editor_dialog_set_event (window->event_editor, event, FALSE);
-  gtk_widget_show (GTK_WIDGET (window->event_editor));
+  gcal_event_widget_show_preview (event_widget, event_preview_cb, user_data);
 }
 
 static void
@@ -797,6 +896,9 @@ gcal_window_finalize (GObject *object)
 
   gcal_clear_date_time (&window->active_date);
 
+  g_clear_object (&window->colors_provider);
+  g_clear_object (&window->provider);
+
   G_OBJECT_CLASS (gcal_window_parent_class)->finalize (object);
 
   GCAL_EXIT;
@@ -830,6 +932,16 @@ gcal_window_constructed (GObject *object)
   g_object_bind_property (self, "context", self->quick_add_popover, "context", G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
   g_object_bind_property (self, "context", self->search_button, "context", G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
 
+  /* CSS */
+  load_css_providers (self);
+
+  g_object_connect (gcal_context_get_manager (self->context),
+                    "swapped-object-signal::calendar-added", recalculate_calendar_colors_css, self,
+                    "swapped-object-signal::calendar-changed", recalculate_calendar_colors_css, self,
+                    "swapped-object-signal::calendar-removed", recalculate_calendar_colors_css, self,
+                    NULL);
+  recalculate_calendar_colors_css (self);
+
   GCAL_EXIT;
 }
 
@@ -1188,3 +1300,19 @@ gcal_window_open_event_by_uuid (GcalWindow  *self,
                                                                  edit_dialog_data);
     }
 }
+
+void
+gcal_window_import_files (GcalWindow  *self,
+                          GFile      **files,
+                          gint         n_files)
+{
+  g_return_if_fail (GCAL_IS_WINDOW (self));
+
+  g_clear_pointer (&self->import_dialog, gtk_widget_destroy);
+
+  self->import_dialog = gcal_import_dialog_new_for_files (self->context, files, n_files);
+  gtk_window_set_transient_for (GTK_WINDOW (self->import_dialog), GTK_WINDOW (self));
+  gtk_window_present (GTK_WINDOW (self->import_dialog));
+
+  g_object_add_weak_pointer (G_OBJECT (self->import_dialog), (gpointer *)&self->import_dialog);
+}
